import { ref, watch } from 'vue';
import { numberComparator, typeAwareComparator } from '@/shared/utils/comparator.util';
import { listColor } from '@/shared/types/color.type';

import UITable from './UITable.vue';

import type { ArgTypes, Meta, StoryObj } from '@storybook/vue3';
import type { DataHeader, DataPaginationSort } from '@/shared/types/data.type';

interface MockItemType {
  id: string;
  title: string;
  datefrom: string;
  dateto: string;
  status: string;
  description: string;
  users: number[];
}

const MockItems: MockItemType[] = [
  {
    id: '0',
    title: 'Crear Portal',
    datefrom: '1/1/2021',
    dateto: '10/1/2021',
    status: 'done',
    description: 'Crear Portal con el framework vue y passar todo el contenido actual de CRM a la nueva plataforma',
    users: [5, 1, 2, 3, 4],
  },
  {
    id: '1',
    title: 'página inicio',
    datefrom: '11/1/2021',
    dateto: '15/1/2021',
    status: 'to do',
    description:
      'Quisque ac dui euismod, commodo tellus in, aliquam lorem. Nam auctor sem sed elementum fringilla. Nulla sit amet risus magna. Etiam est augue, hendrerit',
    users: [2, 4, 5],
  },
  {
    id: '2',
    title: 'página kanban',
    datefrom: '7/1/2021',
    dateto: '14/1/2021',
    status: 'doing',
    description:
      'Quisque ac dui euismod, commodo tellus in, aliquam lorem. Nam auctor sem sed elementum fringilla. Nulla sit amet risus magna. Etiam est augue, hendrerit',
    users: [4, 5],
  },
  {
    id: '3',
    title: 'página detalles',
    datefrom: '9/1/2021',
    dateto: '20/1/2021',
    status: 'done',
    description:
      'Quisque ac dui euismod, commodo tellus in, aliquam lorem. Nam auctor sem sed elementum fringilla. Nulla sit amet risus magna. Etiam est augue, hendrerit',
    users: [0],
  },
  {
    id: '4',
    title: 'formulario',
    datefrom: '10/1/2021',
    dateto: '20/1/2021',
    status: 'backlog',
    description:
      'Quisque ac dui euismod, commodo tellus in, aliquam lorem. Nam auctor sem sed elementum fringilla. Nulla sit amet risus magna. Etiam est augue, hendrerit',
    users: [0],
  },
  {
    id: '6',
    title: 'new task from form',
    datefrom: '17/2/2021',
    dateto: '18/2/2021',
    status: 'doing',
    description: '',
    users: [0],
  },
  {
    id: '7',
    title: 'new task from form 2',
    datefrom: '3/2/2021',
    dateto: '8/2/2021',
    status: 'backlog',
    description: 'pdte',
    users: [],
  },
  {
    id: '8',
    title: 'new task from form done',
    datefrom: '2/2/2021',
    dateto: '2/2/2021',
    status: 'to do',
    description: 'asd',
    users: [2],
  },
  {
    id: '9',
    title: 'Nova tasca',
    datefrom: '3/2/2021',
    dateto: '1/3/2021',
    status: 'backlog',
    description: 'Implementant kanban',
    users: [4],
  },
  {
    id: '10',
    title: 'Griso',
    datefrom: '3/2/2021',
    dateto: '3/2/2021',
    status: 'to do',
    description: '',
    users: [],
  },
  {
    id: '13',
    title: 'jfhfjh',
    datefrom: '3/2/2021',
    dateto: '3/2/2021',
    status: 'to do',
    description: '',
    users: [1],
  },
  {
    id: '14',
    title: 'tasca nova',
    datefrom: '3/2/2021',
    dateto: '3/2/2021',
    status: 'backlog',
    description: '',
    users: [],
  },
  {
    id: '15',
    title: 'test',
    datefrom: '3/2/2021',
    dateto: '3/2/2021',
    status: 'backlog',
    description: '',
    users: [],
  },
  {
    id: '16',
    title: 'Prova des del nou form',
    datefrom: '8/2/2021',
    dateto: '22/2/2021',
    status: 'backlog',
    description: 'Prova des del nou form',
    users: [],
  },
  {
    id: '17',
    title: 'Prova dos sense parse a Task',
    datefrom: '1/3/2021',
    dateto: '2/3/2021',
    status: 'backlog',
    description: 'Prova dos sense parse a Task desc',
    users: [],
  },
  {
    id: '18',
    title: 'Griso Mendez',
    datefrom: '4/2/2021',
    dateto: '4/2/2021',
    status: 'backlog',
    description: '',
    users: [],
  },
];

const MockHeaders: DataHeader<MockItemType>[] = [
  {
    key: 'id',
    title: 'Id',
    hidden: true,
    sortable: true,
  }, {
    key: 'title',
    title: 'Título',
    fixed: true,
    sortable: true,
  }, {
    key: 'datefrom',
    title: 'Desde',
  }, {
    key: 'dateto',
    title: 'Hasta',
  }, {
    key: 'status',
    title: 'Estado',
    sortable: true,
  }, {
    key: 'description',
    title: 'Descripción',
    hidden: false,
  }, {
    key: 'users',
    title: 'Usuarios',
    sortable: true,
    comparator: (a, b, desc) => {
      const aName = a.users.length;
      const bName = b.users.length;
      return numberComparator(aName, bName, desc);
    },
  },
];

const meta = {
  title: 'Components/UITable',
  // eslint-disable-next-line @typescript-eslint/ban-ts-comment
  // @ts-ignore
  component: UITable<MockItemType>,
  tags: ['autodocs'],
  argTypes: {
    name: {
      description: 'A string specifying a name for the table',
      table: {
        type: {
          summary: 'string',
        },
      },
      control: { type: 'text' },
    },
    items: {
      description: 'Provides the list of objects to show as table row.',
      defaultValue: { summary: '[]' },
      table: {
        type: {
          summary: 'T[]',
        },
      },
      control: { type: 'object' },
    },
    headers: {
      description: 'Defines each column and the item key that renders by default at the item cell.',
      defaultValue: { summary: '[]' },
      table: {
        type: {
          summary: 'DataHeader<T>[]',
          detail: `
            {
              key: keyof T;
              title: string;
              fixed?: boolean;
              hidden?: boolean;
              sortable?: boolean;
              comparator?: (a: T, b: T, desc: boolean) => number;
            }
          `,
        },
      },
      control: { type: 'object' },
    },
    dynamicColumns: {
      description: 'Allows to hide columns and change order',
      table: {
        type: {
          summary: 'boolean',
        },
      },
      control: { type: 'boolean' },
    },
    selectable: {
      description: 'Shows a checkbox before each row to select items',
      table: {
        type: {
          summary: 'boolean',
        },
      },
      control: { type: 'boolean' },
    },
    loading: {
      description: 'Renders a loader to let know the user that it\'s loading more items',
      table: {
        type: {
          summary: 'boolean',
        },
      },
      control: { type: 'boolean' },
    },
    handleFromServer: {
      description: 'If set true the pagination and sort is handled by the server and not by the component',
      table: {
        type: {
          summary: 'boolean',
        },
      },
      control: { type: 'boolean' },
    },
    loadingText: {
      description: 'The text that follows the loader to emphasize the action',
      table: {
        type: {
          summary: 'string',
        },
      },
      control: { type: 'text' },
    },
    emptyText: {
      description: 'The text that follows the warning icon to describe that there are no items to display',
      table: {
        type: {
          summary: 'string',
        },
      },
      control: { type: 'text' },
    },
    selectedItems: {
      description: 'List of keys from items that are selected.',
      defaultValue: { summary: 'new Set()' },
      table: {
        type: {
          summary: 'Set<string>',
        },
      },
      control: { type: 'object' },
    },
    highlightedItems: {
      description: 'List of keys from items that are highlighted.',
      defaultValue: { summary: 'new Set()' },
      table: {
        type: {
          summary: 'Set<string>',
        },
      },
      control: { type: 'object' },
    },
    color: {
      description: 'Color from the defined palette',
      options: listColor,
      defaultValue: { summary: 'default' },
      table: {
        type: {
          summary: 'Color',
          detail: '"default" | "primary" | "secondary" | "success" | "warning" | "error"',
        },
      },
      control: { type: 'select' },
    },
    paginationSort: {
      description: 'Defines pagination: actual page, rows per page, total items, paginator texts and sorting',
      defaultValue: { summary: '{ sortBy: undefined, sortDirection: \'asc\', page: 1, rowsPerPage: 0, totalItems: 0, entityText: undefined, entitySeparatorText: \'de\', pageButtonText: \'Ver más\', scrollPosition: 0 }' },
      table: {
        type: {
          summary: 'DataPaginationSort<T>',
          detail: `
            {
              sortBy?: keyof T;
              sortDirection?: 'asc' | 'desc';
              page: number;
              rowsPerPage: number;
              totalItems?: number;
              entityText?: string;
              entitySeparatorText?: string;
              pageButtonText?: string;
              scrollPosition?: number;
            }
          `,
        },
      },
      control: { type: 'object' },
    },
  } as ArgTypes,
  args: {
    name: 'table-name',
    items: MockItems,
    headers: MockHeaders,
    dynamicColumns: false,
    selectable: false,
    loading: false,
    loadingText: '',
    emptyText: '',
    selectedItems: new Set<string>,
    highlightedItems: new Set<string>,
    color: 'default',
    paginationSort: {
      sortBy: undefined,
      sortDirection: 'asc',
      page: 1,
      rowsPerPage: 0,
      entityText: 'items',
      entitySeparatorText: 'de',
      pageButtonText: 'Ver más',
      scrollPosition: 0,
    } as DataPaginationSort<MockItemType>,
    handleFromServer: false,
  },
  render: args => ({
    components: { UITable },
    setup() {
      return { args };
    },
    template:  `
      <UITable
        v-bind="args"
        v-model:headers="args.headers"
        v-model:selectedItems="args.selectedItems"
        v-model:highlightedItems="args.highlightedItems"
        v-model:paginationSort="args.paginationSort"
      />
    `,
  }),
} satisfies Meta<typeof UITable<MockItemType>>;

export default meta;
type Story = StoryObj<typeof UITable<MockItemType>>;

export const Default: Story = {};

export const NoHeaders: Story = {
  args: {
    headers: undefined,
  },
};

export const SelectedItems: Story = {
  args: {
    selectable: true,
    selectedItems: new Set(['0', '2', '4']),
  },
};

export const HighlightedItems: Story = {
  args: {
    highlightedItems: new Set(['0', '2', '4']),
  },
};

export const ColorPrimary: Story = {
  args: {
    color: 'primary',
  },
};

export const ColorSecondary: Story = {
  args: {
    color: 'secondary',
  },
};

export const DynamicColumns: Story = {
  args: {
    dynamicColumns: true,
  },
};

export const Loading: Story = {
  args: {
    items: [],
    loading: true,
    loadingText: 'Cargando',
  },
};

export const LoadingMore: Story = {
  args: {
    paginationSort: {
      sortBy: undefined,
      sortDirection: 'asc',
      page: 1,
      rowsPerPage: 5,
      entityText: 'items',
      entitySeparatorText: 'de',
      pageButtonText: 'Ver más',
      scrollPosition: 0,
    },
    loading: true,
    loadingText: 'Cargando',
  },
};

export const Empty: Story = {
  args: {
    items: [],
    emptyText: 'Sin datos',
  },
};

export const RowsPerPage5: Story = {
  args: {
    paginationSort: {
      sortBy: undefined,
      sortDirection: 'asc',
      page: 1,
      rowsPerPage: 5,
      entityText: 'items',
      entitySeparatorText: 'de',
      pageButtonText: 'Ver más',
      scrollPosition: 0,
    },
  },
};

const serverItems = ref(MockItems.slice(0, 5));
const serverLoading = ref(false);
const serverPaginationSort = ref<DataPaginationSort<MockItemType>>({
  sortBy: undefined,
  sortDirection: 'asc',
  page: 1,
  rowsPerPage: 5,
  totalItems: 100,
  entityText: 'items',
  entitySeparatorText: 'de',
  pageButtonText: 'Ver más',
  scrollPosition: 0,
});

watch(serverPaginationSort, async (newVal, oldVal) => {
  serverLoading.value = true;
  if (newVal.sortBy !== oldVal?.sortBy || newVal.sortDirection !== oldVal?.sortDirection) {
    serverItems.value = [];
    serverItems.value = await getServerData(newVal);
  } else if (newVal.page !== oldVal.page) {
    const moreItems = await getServerData(newVal);
    serverItems.value = [...serverItems.value, ...moreItems];
  }
  serverLoading.value = false;
});

async function getServerData(paginationSort: DataPaginationSort<MockItemType>) {
  // Mock de llamada a api: Esta funcion deberia llamar a un endpoint con los datos de paginationSort para que el servidor devuelva los datos de la pagina ya ordenados
  // const result = api.get<MockItemType[]>('items', paginationSort);
  // Mock - ini
  const comparator = MockHeaders.find(x => x.key === paginationSort.sortBy)?.comparator;
  let compareFn: ((a: MockItemType, b: MockItemType) => number) | undefined;
  if (comparator) {
    compareFn = (a: MockItemType, b: MockItemType) => comparator(a, b, paginationSort.sortDirection === 'desc');
  } else {
    compareFn = (a: MockItemType, b: MockItemType) => typeAwareComparator(Reflect.get(a, paginationSort.sortBy as string), Reflect.get(b, paginationSort.sortBy as string), paginationSort.sortDirection === 'desc');
  }
  await new Promise(resolve => setTimeout(resolve, 3000));
  serverPaginationSort.value.totalItems = 100;
  const itemFrom = (paginationSort.page - 1) * paginationSort.rowsPerPage;
  const result = [...MockItems].sort(compareFn).slice(itemFrom, itemFrom + paginationSort.rowsPerPage);
  // Mock - fin
  return result;
}

export const HandleFromServer: Story = {
  args: {
    handleFromServer: true,
    paginationSort: {
      sortBy: undefined,
      sortDirection: 'asc',
      page: 1,
      rowsPerPage: 5,
      totalItems: 100,
      entityText: 'items',
      entitySeparatorText: 'de',
      pageButtonText: 'Ver más',
      scrollPosition: 0,
    },
  },
  render: args => ({
    components: { UITable },
    setup() {
      return { args, serverItems, serverLoading, serverPaginationSort };
    },
    template:  `
      <UITable
        v-bind="args"
        :items="serverItems"
        :loading="serverLoading"
        v-model:headers="args.headers"
        v-model:selectedItems="args.selectedItems"
        v-model:highlightedItems="args.highlightedItems"
        v-model:paginationSort="serverPaginationSort"
      />
    `,
  }),
};

export const ScrollInitPageTwo: Story = {
  args: {
    dynamicColumns: true,
    paginationSort: {
      sortBy: undefined,
      sortDirection: 'asc',
      page: 2,
      rowsPerPage: 5,
      entityText: 'items',
      entitySeparatorText: 'de',
      pageButtonText: 'Ver más',
      scrollPosition: 60,
    },
  },
  render: args => ({
    components: { UITable },
    setup() {
      return { args };
    },
    template:  `
      <UITable
        v-bind="args"
        v-model:headers="args.headers"
        v-model:paginationSort="args.paginationSort"
      >
        <template #append-cell>
          <span>fixed</span>
        </template>
      </UITable>
    `,
  }),
};

export const WithSlots: Story = {
  render: args => ({
    components: { UITable },
    setup() {
      return { args };
    },
    template:  `
      <UITable
        v-bind="args"
        v-model:headers="args.headers"
        v-model:paginationSort="args.paginationSort"
      >
        <template #message-content>
          <span>This space is reserved between the table and the paginator</span>
        </template>
        <template #append-cell="{item}">
          <span>{{item.id}}</span>
        </template>
        <template #status-cell="{item}">
          <span v-if="item.status === 'done'" class="color--success">{{ item.status }}</span>
          <span v-else-if="item.status === 'to do'" class="color--warning">{{ item.status }}</span>
          <span v-else-if="item.status === 'doing'" class="color--secondary">{{ item.status }}</span>
        </template>
        <template #users-cell="{item}">
          <span v-if="item.users.length > 0" class="color--secondary">{{ item.users.length }} users</span>
          <span v-else class="color--error">{{ item.users.length }} users</span>
        </template>
      </UITable>
    `,
  }),
};